@@ -1,3 +1,5 @@
+#' @include defaults.R
+
 # record input/output lines numbers in Rnw/tex and filenames
 knit_concord = new_defaults(list(
   inlines = NULL, outlines = NULL, infile = NULL, outfile = NULL
@@ -12,8 +14,8 @@ concord_mode = function() {
 current_lines = function(i) {
   # a helpr function to return line numbers for block i
   n = knit_concord$get('inlines')
-  n0 = sum(head(n, i - 1L)) + 1L; n1 = n0 + n[i] - 1L
-  c(n0, n1)
+  n1 = sum(head(n, i)); n0 = n1 - n[i] + 2
+  c(min(n0, n1), n1)
 }
 
 ## generate concordance for RStudio
