@@ -35,8 +35,7 @@ hook_plot_html = function(x, options) {
 
 ## a wrapper to upload an image and return the URL
 .upload.url = function(x) {
-  file = paste(x, collapse = '.')
-  opts_knit$get('upload.fun')(file)
+  opts_knit$get('upload.fun')(x)
 }
 
 .chunk.hook.html = function(x, options) {
@@ -65,12 +64,11 @@ hook_plot_html = function(x, options) {
 #' @rdname hook_animation
 #' @export
 hook_ffmpeg_html = function(x, options) {
+  x = c(sans_ext(x), file_ext(x))
   fig.num = options$fig.num
   # set up the ffmpeg run
-  ffmpeg.opts = options$aniopts
   fig.fname = str_c(sub(str_c(fig.num, '$'), '%d', x[1]), '.', x[2])
   mov.fname = str_c(sub(paste(fig.num, '$',sep = ''), '', x[1]), '.ogg')
-  if(is.na(ffmpeg.opts)) ffmpeg.opts = NULL
 
   ffmpeg.cmd = paste('ffmpeg', '-y', '-r', 1/options$interval,
                      '-i', fig.fname, mov.fname)
@@ -93,6 +91,7 @@ opts_knit$set(animation.fun = hook_ffmpeg_html)
 #' @rdname hook_animation
 #' @export
 hook_scianimator = function(x, options) {
+  x = c(sans_ext(x), file_ext(x))
   fig.num = options$fig.num
   base = opts_knit$get('base.url') %n% ''
 
@@ -130,6 +129,7 @@ hook_scianimator = function(x, options) {
 #' @rdname hook_animation
 #' @export
 hook_r2swf = function(x, options) {
+  x = c(sans_ext(x), file_ext(x))
   fig.num = options$fig.num
   # set up the R2SWF run
   fig.name = str_c(sub(str_c(fig.num, '$'), '', x[1]), 1:fig.num, '.', x[2])
@@ -149,16 +149,14 @@ hook_r2swf = function(x, options) {
 #' @rdname output_hooks
 #' @export
 render_html = function() {
-  knit_hooks$restore()
   set_html_dev()
   opts_knit$set(out.format = 'html')
   ## use div with different classes
   html.hook = function(name) {
     force(name)
     function (x, options) {
-      if (name == 'source') {
-        x = paste(c(hilight_source(x, 'html', options), ''), collapse = '\n')
-      }
+      if (name == 'source') x = c(hilight_source(x, 'html', options), '')
+      x = paste(x, collapse = '\n')
       sprintf('<div class="%s"><pre class="knitr %s">%s</pre></div>\n', name, tolower(options$engine), x)
     }
   }
@@ -171,7 +169,5 @@ render_html = function() {
   knit_hooks$set(inline = function(x) {
     sprintf(if (inherits(x, 'AsIs')) '%s' else '<code class="knitr inline">%s</code>',
             .inline.hook(format_sci(x, 'html')))
-  }, output = function(x, options) {
-    if (output_asis(x, options)) x else html.hook('output')(x, options)
-  }, plot = hook_plot_html, chunk = .chunk.hook.html)
+  }, output = html.hook('output'), plot = hook_plot_html, chunk = .chunk.hook.html)
 }
