@@ -116,8 +116,11 @@ knit = function(input, output = NULL, tangle = FALSE, text = NULL, quiet = FALSE
     # in child mode, input path needs to be adjusted
     if (in.file && !is_abs_path(input)) {
       input = paste(opts_knit$get('child.path'), input, sep = '')
-      input = file.path(input_dir(), input)
+      input = input2 = file.path(input_dir(), input)
     }
+    # respect the quiet argument in child mode (#741)
+    optk = opts_knit$get(); on.exit(opts_knit$set(optk), add = TRUE)
+    opts_knit$set(progress = opts_knit$get('progress') && !quiet)
   } else {
     oenvir = .knitEnv$knit_global; .knitEnv$knit_global = envir
     on.exit({.knitEnv$knit_global = oenvir}, add = TRUE)
@@ -127,15 +130,13 @@ knit = function(input, output = NULL, tangle = FALSE, text = NULL, quiet = FALSE
     adjust_opts_knit()
     ## turn off fancy quotes, use smaller digits/width, warn immediately
     oopts = options(
-      useFancyQuotes = FALSE, digits = 4L, warn = 1L, width = opts_knit$get('width'),
-      knitr.in.progress = TRUE,
-      device = function(width = 7, height = 7, ...) pdf(NULL, width, height, ...)
+      useFancyQuotes = FALSE, digits = 4L, width = opts_knit$get('width'),
+      knitr.in.progress = TRUE, device = pdf_null
     )
     on.exit(options(oopts), add = TRUE)
     # restore chunk options after parent exits
     optc = opts_chunk$get(); on.exit(opts_chunk$restore(optc), add = TRUE)
     ocode = knit_code$get(); on.exit(knit_code$restore(ocode), add = TRUE)
-    if (tangle) knit_code$restore() # clean up code before tangling
     optk = opts_knit$get(); on.exit(opts_knit$set(optk), add = TRUE)
     opts_knit$set(tangle = tangle, encoding = encoding,
                   progress = opts_knit$get('progress') && !quiet
@@ -149,7 +150,20 @@ knit = function(input, output = NULL, tangle = FALSE, text = NULL, quiet = FALSE
     ext = tolower(file_ext(input))
     if ((is.null(output) || is.na(output)) && !child_mode())
       output = basename(auto_out_name(input, ext))
-    options(tikzMetricsDictionary = tikz_dict(input)) # cache tikz dictionary
+    # do not run purl() when the output is newer than input (the output might
+    # have been generated by hook_purl)
+    if (is.character(output) && !child_mode()) {
+      out.purl = sub_ext(input, 'R')
+      if (tangle && file_test('-nt', out.purl, input)) return(out.purl)
+      otangle = .knitEnv$tangle.file  # the tangled R script
+      .knitEnv$tangle.file = normalizePath(out.purl, mustWork = FALSE)
+      .knitEnv$tangle.start = FALSE
+      on.exit({.knitEnv$tangle.file = otangle; .knitEnv$tangle.start = NULL}, add = TRUE)
+    }
+    if (is.null(getOption('tikzMetricsDictionary'))) {
+      options(tikzMetricsDictionary = tikz_dict(input)) # cache tikz dictionary
+      on.exit(options(tikzMetricsDictionary = NULL), add = TRUE)
+    }
     knit_concord$set(infile = input, outfile = output)
   }
 
@@ -177,17 +191,18 @@ knit = function(input, output = NULL, tangle = FALSE, text = NULL, quiet = FALSE
            ext, "' in built-in pattern lists; ",
            'see ?knit_patterns on how to set up customized patterns')
     set_pattern(pattern)
-    if (pattern == 'rnw' && is_sweave(text)) remind_sweave(if (in.file) input)
+    if (pattern == 'rnw' && length(sweave_lines <- which_sweave(text)) > 0)
+      remind_sweave(if (in.file) input, sweave_lines)
     opts_knit$set(out.format = switch(
       pattern, rnw = 'latex', tex = 'latex', html = 'html', md = 'markdown',
-      rst = 'rst', brew = 'brew', asciidoc = 'asciidoc'
+      rst = 'rst', brew = 'brew', asciidoc = 'asciidoc', textile = 'textile'
     ))
   }
 
   if (is.null(out_format())) auto_format(ext)
   ## change output hooks only if they are not set beforehand
   if (identical(knit_hooks$get(names(.default.hooks)), .default.hooks) && !child_mode()) {
-    do.call(paste('render', out_format(), sep = '_'), list(), envir = getNamespace('knitr'))
+    getFromNamespace(paste('render', out_format(), sep = '_'), 'knitr')()
     on.exit(knit_hooks$set(.default.hooks), add = TRUE)
   }
 
@@ -218,8 +233,6 @@ knit = function(input, output = NULL, tangle = FALSE, text = NULL, quiet = FALSE
 #' @param ... arguments passed to \code{\link{knit}()} from \code{purl()}
 #' @export
 purl = function(..., documentation = 1L) {
-  # for compatibility to knitr <= 0.8.8
-  if (is.logical(documentation)) documentation = ifelse(documentation, 2L, 1L)
   doc = opts_knit$get('documentation'); on.exit(opts_knit$set(documentation = doc))
   opts_knit$set(documentation = documentation)
   knit(..., tangle = TRUE)
@@ -270,7 +283,7 @@ auto_out_name = function(input, ext = tolower(file_ext(input))) {
   base = sans_ext(input)
   name = if (opts_knit$get('tangle')) c(base, '.R') else
     if (ext %in% c('rnw', 'snw')) c(base, '.tex') else
-      if (ext %in% c('rmd', 'rmarkdown', 'rhtml', 'rhtm', 'rtex', 'stex', 'rrst'))
+      if (ext %in% c('rmd', 'rmarkdown', 'rhtml', 'rhtm', 'rtex', 'stex', 'rrst', 'rtextile'))
         c(base, '.', substring(ext, 2L)) else
           if (grepl('_knit_', input)) sub('_knit_', '', input) else
             if (ext != 'txt') c(base, '.txt') else c(base, '-out.', ext)
@@ -367,29 +380,48 @@ knit_log = new_defaults()  # knitr log for errors, warnings and messages
 #' @param x output from \code{\link[evaluate]{evaluate}}
 #' @param options list of options used to control output
 #' @noRd
-#' @S3method wrap list
-#' @S3method wrap character
-#' @S3method wrap source
-#' @S3method wrap warning
-#' @S3method wrap message
-#' @S3method wrap error
-#' @S3method wrap recordedplot
 wrap = function(x, options = list()) {
   UseMethod('wrap', x)
 }
 
+#' @export
 wrap.list = function(x, options = list()) {
   if (length(x) == 0L) return(x)
   lapply(x, wrap, options)
 }
 
+# ignore unknown classes
+#' @export
+wrap.default = function(x, options) return()
+
+#' @export
 wrap.character = function(x, options) {
-  if (!output_asis(x, options)) x = comment_out(x, options$comment)
+  if (output_asis(x, options)) {
+    if (!out_format('latex')) return(x)  # latex output still need a tweak
+  } else x = comment_out(x, options$comment)
   knit_hooks$get('output')(x, options)
 }
 
+# if you provide a custom print function that returns a character object of
+# class 'knit_asis', I'll just write it as is
+#' @export
+wrap.knit_asis = function(x, options) {
+  if (isFALSE(attr(x, 'knit_cacheable', exact = TRUE)) &&
+        (!missing(options) && options$cache > 0))
+    stop("The code chunk '", options$label, "' is not cacheable; ",
+         "please use the chunk option cache=FALSE on this chunk")
+  m = attr(x, 'knit_meta', exact = TRUE)
+  if (length(m)) {
+    .knitEnv$meta = c(.knitEnv$meta, m)
+  }
+  as.character(x)
+}
+
+#' @export
 wrap.source = function(x, options) {
   src = sub('\n$', '', x$src)
+  if (!options$collapse && options$strip.white) src = strip_white(src)
+  if (is_blank(src)) return()  # an empty chunk
   knit_hooks$get('source')(src, options)
 }
 
@@ -404,18 +436,22 @@ msg_wrap = function(message, type, options) {
   knit_hooks$get(type)(comment_out(message, options$comment), options)
 }
 
+#' @export
 wrap.warning = function(x, options) {
   msg_wrap(paste('Warning:', x$message, collapse = '\n'), 'warning', options)
 }
 
+#' @export
 wrap.message = function(x, options) {
   msg_wrap(paste(x$message, collapse = ''), 'message', options)
 }
 
+#' @export
 wrap.error = function(x, options) {
   msg_wrap(str_c('Error: ', x$message), 'error', options)
 }
 
+#' @export
 wrap.recordedplot = function(x, options) {
   ## figure number sequence for multiple plots
   fig.cur = plot_counter()
@@ -424,10 +460,119 @@ wrap.recordedplot = function(x, options) {
   if (!file.exists(dirname(name)))
     dir.create(dirname(name), recursive = TRUE) # automatically creates dir for plots
   ## vectorize over dev, ext and dpi: save multiple versions of the plot
-  name.ext = mapply(save_plot, width = options$fig.width, height = options$fig.height,
-                    dev = options$dev, ext = options$fig.ext, dpi = options$dpi,
-                    MoreArgs = list(plot = x, name = name, options = options),
-                    SIMPLIFY = FALSE)[[1]]
+  file = mapply(
+    save_plot, width = options$fig.width, height = options$fig.height,
+    dev = options$dev, ext = options$fig.ext, dpi = options$dpi,
+    MoreArgs = list(plot = x, name = name, options = options), SIMPLIFY = FALSE
+  )[[1]]
   if (options$fig.show == 'hide') return('')
-  knit_hooks$get('plot')(name.ext, reduce_plot_opts(options))
+  knit_hooks$get('plot')(file, reduce_plot_opts(options))
+}
+
+#' A custom printing function
+#'
+#' The S3 generic function \code{knit_print} is the default printing function in
+#' \pkg{knitr}. The chunk option \code{render} uses this function by default.
+#' The main purpose of this S3 generic function is to customize printing of R
+#' objects in code chunks. We can fall back to the normal printing behavior by
+#' setting the chunk option \code{render = normal_print}.
+#'
+#' Users can write custom methods based on this generic function. For example,
+#' if we want to print all data frames as tables in the output, we can define a
+#' method \code{knit_print.data.frame} that turns a data frame into a table (the
+#' implementation may use other R packages or functions, e.g. \pkg{xtable} or
+#' \code{\link{kable}()}).
+#' @param x an R object to be printed
+#' @param ... additional arguments passed to the S3 method (currently ignored,
+#'   except two optional arguments \code{options} and \code{inline}; see
+#'   References)
+#' @return The value returned from the print method should be a character vector
+#'   or can be converted to a character value. You can wrap the value in
+#'   \code{\link{asis_output}()} so that \pkg{knitr} writes the character value
+#'   as is in the output.
+#' @note It is recommended to leave a \code{...} argument in your method, to
+#'   allow future changes of the \code{knit_print()} API without breaking your
+#'   method.
+#' @references See \code{vignette('knit_print', package = 'knitr')}.
+#' @export
+#' @examples library(knitr)
+#' # write tables for data frames
+#' knit_print.data.frame = function(x, ...) {
+#'   res = paste(c('', '', kable(x, output = FALSE)), collapse = '\n')
+#'   asis_output(res)
+#' }
+#' # after you defined the above method, data frames will be printed as tables in knitr,
+#' # which is different with the default print() behavior
+knit_print = function(x, ...) {
+  UseMethod('knit_print', x)
+}
+
+#" the default print method is just print()/show()
+#' @export
+knit_print.default = function(x, ..., inline = FALSE) {
+  if (inline) x else normal_print(x)
+}
+
+#' @export
+knit_print.knit_asis = function(x, ...) x
+
+#' @rdname knit_print
+#' @export
+normal_print = default_handlers$value
+
+#' Mark an R object with a special class
+#'
+#' This is a convenience function that assigns the input object a class named
+#' \code{knit_asis}, so that \pkg{knitr} will treat it as is (the effect is the
+#' same as the chunk option \code{results = 'asis'}) when it is written to the
+#' output.
+#'
+#' This function is normally used in a custom S3 method based on the printing
+#' function \code{\link{knit_print}()}.
+#'
+#' For the \code{cacheable} argument, you need to be careful when printing the
+#' object involves non-trivial side effects, in which case it is strongly
+#' recommended to use \code{cacheable = FALSE} to instruct \pkg{knitr} that this
+#' object should not be cached using the chunk option \code{cache = TRUE},
+#' otherwise the side effects will be lost the next time the chunk is knitted.
+#' For example, printing a \pkg{shiny} input element in an R Markdown document
+#' may involve registering metadata about some JavaScript libraries or
+#' stylesheets, and the metadata may be lost if we cache the code chunk, because
+#' the code evaluation will be skipped the next time.
+#' @param x an R object (typically a character string, or can be converted to a
+#'   character string via \code{\link{as.character}()})
+#' @param meta additional metadata of the object to be printed (the metadata
+#'   will be collected when the object is printed, and accessible via
+#'   \code{knit_meta()})
+#' @param cacheable a logical value indicating if this object is cacheable
+#' @export
+#' @examples  # see ?knit_print
+asis_output = function(x, meta = NULL, cacheable = length(meta) == 0) {
+  structure(x, class = 'knit_asis', knit_meta = meta, knit_cacheable = cacheable)
+}
+
+#' Metadata about objects to be printed
+#'
+#' As an object is printed, \pkg{knitr} will collect metadata about it (if
+#' available). After knitting is done, all the metadata is accessible via this
+#' function.
+#' @param class optionally return only metadata entries that inherit from the
+#'   specified class; the default, \code{NULL}, returns all entries.
+#' @param clean whether to clean the collected metadata; by default, the
+#'   metadata stored in \pkg{knitr} is cleaned up once retrieved, because we may
+#'   not want the metadata to be passed to the next \code{knit()} call; to be
+#'   defensive (i.e. not to have carryover metadata), you can call
+#'   \code{knit_meta()} before \code{knit()}
+#' @export
+knit_meta = function(class = NULL, clean = TRUE) {
+  matches = if (is.null(class)) {
+    # if no class was specified, match the whole list
+    seq_along(.knitEnv$meta)
+  } else if (length(.knitEnv$meta)) {
+    # if a class was specified, match the items belonging to the class
+    which(sapply(.knitEnv$meta, inherits, what = class))
+  }
+  if (length(matches) < 1) return(list())
+  if (clean) on.exit(.knitEnv$meta[matches] <- NULL, add = TRUE)
+  .knitEnv$meta[matches]
 }
