@@ -1,6 +1,6 @@
 #' Create tables in LaTeX, HTML, Markdown and reStructuredText
 #'
-#' This is a very simple table generator. It is simply by design. It is not
+#' This is a very simple table generator. It is simple by design. It is not
 #' intended to replace any other R packages for making tables.
 #' @param x an R object (typically a matrix or data frame)
 #' @param format a character string; possible values are \code{latex},
@@ -8,7 +8,8 @@
 #'   automatically determined if the function is called within \pkg{knitr}; it
 #'   can also be set in the global option \code{knitr.table.format}
 #' @param digits the maximum number of digits for numeric columns (passed to
-#'   \code{round()})
+#'   \code{round()}); it can also be a vector of length \code{ncol(x)} to set
+#'   the number of digits for individual columns
 #' @param row.names whether to include row names; by default, row names are
 #'   included if they are neither \code{NULL} nor identical to \code{1:nrow(x)}
 #' @param align the alignment of columns: a character vector consisting of
@@ -19,13 +20,23 @@
 #' @param ... other arguments (see examples)
 #' @return A character vector of the table source code. When \code{output =
 #'   TRUE}, the results are also written into the console as a side-effect.
-#' @seealso Other R packages such as \pkg{xtable} and \pkg{tables}.
+#' @seealso Other R packages such as \pkg{xtable} and \pkg{tables} for HTML and
+#'   LaTeX tables, and \pkg{ascii} and \pkg{pander} for different flavors of
+#'   markdown output and some advanced features and table styles.
 #' @note The tables for \code{format = 'markdown'} also work for Pandoc when the
 #'   \code{pipe_tables} extension is enabled (this is the default behavior for
 #'   Pandoc >= 1.10).
+#'
+#'   When using this function inside a \pkg{knitr} document (e.g. R Markdown or
+#'   R LaTeX), you will need the chunk option \code{results='asis'}.
+#' @references See
+#'   \url{https://github.com/yihui/knitr-examples/blob/master/091-knitr-table.Rnw}
+#'    for some examples in LaTeX, but they also apply to other document formats.
 #' @export
-#' @examples kable(head(iris), format = 'latex')
+#' @examples  kable(head(iris), format = 'latex')
 #' kable(head(iris), format = 'html')
+#' kable(head(iris), format = 'latex', caption = 'Title of the table')
+#' kable(head(iris), format = 'html', caption = 'Title of the table')
 #' # use the booktabs package
 #' kable(mtcars, format = 'latex', booktabs = TRUE)
 #' # use the longtable package
@@ -38,14 +49,18 @@
 #' kable(head(mtcars), format = 'rst', row.names = FALSE)
 #' # R Markdown/Github Markdown tables
 #' kable(head(mtcars[, 1:5]), format = 'markdown')
+#' # no inner padding
+#' kable(head(mtcars), format = 'markdown', padding = 0)
+#' # more padding
+#' kable(head(mtcars), format = 'markdown', padding = 2)
 #' # Pandoc tables
-#' kable(head(mtcars), format = 'pandoc')
+#' kable(head(mtcars), format = 'pandoc', caption = 'Title of the table')
 #' # save the value
 #' x = kable(mtcars, format = 'html', output = FALSE)
 #' cat(x, sep = '\n')
 #' # can also set options(knitr.table.format = 'html') so that the output is HTML
 kable = function(x, format, digits = getOption('digits'), row.names = NA,
-                 align, output = TRUE, ...) {
+                 align, output = getOption('knitr.table.output', TRUE), ...) {
   if (missing(format)) format = getOption('knitr.table.format', switch(
     out_format() %n% 'markdown', latex = 'latex', listings = 'latex', sweave = 'latex',
     html = 'html', markdown = 'markdown', rst = 'rst',
@@ -54,25 +69,32 @@ kable = function(x, format, digits = getOption('digits'), row.names = NA,
   # if the original object does not have colnames, we need to remove them later
   ncn = is.null(colnames(x))
   if (!is.matrix(x) && !is.data.frame(x)) x = as.data.frame(x)
+  m = ncol(x)
   # numeric columns
-  isn = if (is.matrix(x)) rep(is.numeric(x), ncol(x)) else sapply(x, is.numeric)
+  isn = if (is.matrix(x)) rep(is.numeric(x), m) else sapply(x, is.numeric)
   if (missing(align) || (format == 'latex' && is.null(align)))
     align = ifelse(isn, 'r', 'l')
   # rounding
-  x = apply(x, 2, function(z) if (is.numeric(z)) format(round(z, digits)) else z)
-  if (is.null(dim(x))) x = t(as.matrix(x))  # damn it!
+  digits = rep(digits, length.out = m)
+  for (j in seq_len(m)) {
+    if (is.numeric(x[, j])) x[, j] = round(x[, j], digits[j])
+  }
+  if (any(isn)) x[, isn] = format(x[, isn], trim = TRUE)
   if (is.na(row.names))
     row.names = !is.null(rownames(x)) && !identical(rownames(x), as.character(seq_len(NROW(x))))
+  if (!is.null(align)) align = rep(align, length.out = m)
   if (row.names) {
     x = cbind(' ' = rownames(x), x)
     if (!is.null(align)) align = c('l', align)  # left align row names
   }
-  x = as.matrix(x)
+  x = format(as.matrix(x), trim = TRUE, justify = 'none')
   if (ncn) colnames(x) = NULL
-  if (!is.null(align)) align = rep(align, length.out = ncol(x))
   attr(x, 'align') = align
   res = do.call(paste('kable', format, sep = '_'), list(x = x, ...))
-  if (output) cat(res, sep = '\n')
+  if (output) {
+    if (!(format %in% c('html', 'latex'))) cat('\n\n')
+    cat(res, sep = '\n')
+  }
   invisible(res)
 }
 
@@ -80,36 +102,53 @@ kable_latex = function(
   x, booktabs = FALSE, longtable = FALSE,
   vline = if (booktabs) '' else '|',
   toprule = if (booktabs) '\\toprule' else '\\hline',
-  bottomrule = if (booktabs) '\\bottomrule' else '\\hline'
+  bottomrule = if (booktabs) '\\bottomrule' else '\\hline',
+  midrule = if (booktabs) '\\midrule' else '\\hline',
+  linesep = if (booktabs) c('', '', '', '', '\\addlinespace') else '\\hline',
+  caption = NULL
 ) {
-  if (!is.null(align <- attr(x, 'align'))) {
+  if (!is.null(align <- attr(x, 'align', exact = TRUE))) {
     align = paste(align, collapse = vline)
     align = paste('{', align, '}', sep = '')
   }
+  cap = if (is.null(caption)) '' else sprintf('\n\\caption{%s}', caption)
+
+  if (nrow(x) == 0) midrule = ""
+
+  linesep = if (nrow(x) > 1) {
+    c(rep(linesep, length.out = nrow(x) - 2), linesep[[1L]], '')
+  } else rep('', nrow(x))
+  linesep = ifelse(linesep == "", linesep, paste('\n', linesep, sep = ''))
 
   paste(c(
+    cap,
     sprintf('\n\\begin{%s}', if (longtable) 'longtable' else 'tabular'), align,
     sprintf('\n%s', toprule), '\n',
-    paste(c(if (!is.null(cn <- colnames(x))) paste(cn, collapse = ' & '),
-            apply(x, 1, paste, collapse = ' & ')),
-          collapse = sprintf('\\\\\n%s\n', if (booktabs) '\\midrule' else '\\hline')),
-    sprintf('\\\\\n%s', bottomrule),
+    if (!is.null(cn <- colnames(x)))
+      paste(paste(cn, collapse = ' & '), sprintf('\\\\\n%s\n', midrule), sep = ''),
+    paste(apply(x, 1, paste, collapse = ' & '), sprintf('\\\\%s', linesep),
+          sep = '', collapse = '\n'),
+    sprintf('\n%s', bottomrule),
     sprintf('\n\\end{%s}', if (longtable) 'longtable' else 'tabular')
   ), collapse = '')
 }
 
-kable_html = function(x, table.attr = '') {
+kable_html = function(x, table.attr = '', caption = NULL) {
   table.attr = gsub('^\\s+|\\s+$', '', table.attr)
   # need a space between <table and attributes
   if (nzchar(table.attr)) table.attr = paste('', table.attr)
+  align = if (is.null(align <- attr(x, 'align', exact = TRUE))) '' else {
+    sprintf(' align="%s"', c(l = 'left', c = 'center', r = 'right')[align])
+  }
+  cap = if (is.null(caption)) '' else sprintf('\n<caption>%s</caption>', caption)
   paste(c(
-    sprintf('<table%s>', table.attr),
+    sprintf('<table%s>%s', table.attr, cap),
     if (!is.null(cn <- colnames(x)))
-      c(' <thead>', '  <tr>', paste('   <th>', cn, '</th>'), '  </tr>', ' </thead>'),
+      c(' <thead>', '  <tr>', sprintf('   <th%s> %s </th>', align, cn), '  </tr>', ' </thead>'),
     '<tbody>',
     paste(
       '  <tr>',
-      apply(x, 1, function(z) paste('   <td>', z, '</td>', collapse = '\n')),
+      apply(x, 1, function(z) paste(sprintf('   <td%s> %s </td>', align, z), collapse = '\n')),
       '  </tr>', sep = '\n'
     ),
     '</tbody>',
@@ -125,31 +164,42 @@ kable_html = function(x, table.attr = '') {
 #'   before the header, after the header and at the end of the table,
 #'   respectively
 #' @param sep.col the column separator
+#' @param padding the number of spaces for the table cell padding
 #' @param align.fun a function to process the separator under the header
 #'   according to alignment
 #' @return A character vector of the table content.
 #' @noRd
-kable_mark = function(x, sep.row = c('=', '=', '='), sep.col = '  ',
-                      align.fun = function(s, a) s) {
+kable_mark = function(x, sep.row = c('=', '=', '='), sep.col = '  ', padding = 0,
+                      align.fun = function(s, a) s, rownames.name = '') {
+  # when the column separator is |, replace existing | with its HTML entity
+  if (sep.col == '|') for (j in seq_len(ncol(x))) {
+    x[, j] = gsub('\\|', '&#124;', x[, j])
+  }
   l = apply(x, 2, function(z) max(nchar(z), na.rm = TRUE))
   cn = colnames(x)
   if (!is.null(cn)) {
-    if (grepl('^\\s*$', cn[1L])) cn[1L] = 'id'  # no empty cells
+    if (sep.col == '|') cn = gsub('\\|', '&#124;', cn)
+    if (grepl('^\\s*$', cn[1L])) cn[1L] = rownames.name  # no empty cells for reST
     l = pmax(l, nchar(cn))
   }
-  if (!is.null(align <- attr(x, 'align'))) l = l + 2
+  padding = padding * if (is.null(align <- attr(x, 'align', exact = TRUE))) 2 else {
+    ifelse(align == 'c', 2, 1)
+  }
+  l = pmax(l + padding, 3)  # at least of width 3 for Github Markdown
   s = sapply(l, function(i) paste(rep(sep.row[2], i), collapse = ''))
   res = rbind(if (!is.na(sep.row[1])) s, cn, align.fun(s, align),
               x, if (!is.na(sep.row[3])) s)
   apply(mat_pad(res, l, align), 1, paste, collapse = sep.col)
 }
 
-kable_rst = kable_mark
+kable_rst = function(x, rownames.name = '\\') {
+  kable_mark(x, rownames.name = rownames.name)
+}
 
 # actually R Markdown
-kable_markdown = function(x) {
+kable_markdown = function(x, padding = 1) {
   if (is.null(colnames(x))) stop('the table must have a header (column names)')
-  res = kable_mark(x, c(NA, '-', NA), '|', align.fun = function(s, a) {
+  res = kable_mark(x, c(NA, '-', NA), '|', padding, align.fun = function(s, a) {
     if (is.null(a)) return(s)
     r = c(l = '^.', c = '^.|.$', r = '.$')
     for (i in seq_along(s)) {
@@ -160,8 +210,9 @@ kable_markdown = function(x) {
   sprintf('|%s|', res)
 }
 
-kable_pandoc = function(x) {
-  kable_mark(x, c(NA, '-', if (is.null(colnames(x))) '-' else NA), '  ')
+kable_pandoc = function(x, caption = NULL, padding = 1) {
+  tab = kable_mark(x, c(NA, '-', if (is.null(colnames(x))) '-' else NA), padding = padding)
+  if (is.null(caption)) tab else c(paste('Table:', caption), "", tab)
 }
 
 # pad a matrix
