@@ -1,5 +1,4 @@
 library(testit)
-message('testing closures created in new_defaults()')
 
 z = new_defaults(list(a = 1))
 
@@ -21,6 +20,13 @@ assert(
   identical(z$get(default = TRUE), list(a=1))
 )
 
+assert(
+  '$get(names) keeps the names if drop = FALSE and one or more names do not exist',
+  identical(z$get(c('a', 'c')), list(a = 2, c = NULL)),
+  identical(z$get('c'), NULL),
+  identical(z$get('c', drop = FALSE), list(c = NULL))
+)
+
 assert(
   '$restore() restores to the initial value',
   identical({z$restore(); z$get()}, list(a=1))
