@@ -23,12 +23,33 @@ assert(
   identical(format_sci(NA_real_), NA_character_)
 )
 
+assert(
+  'format_sci() coerces non-numeric and non-double values to characters',
+  format_sci(Sys.Date()) == as.character(Sys.Date()),
+  format_sci(1000000L) == '1000000'
+)
+
+assert(
+  'format_sci() for Rnw does not add \\ensuremath{} at all',
+  !grepl('[\\]ensuremath', format_sci(c(1e4, 1.2345e10, 2*pnorm(-(3:4)), -Inf)))
+)
+
 assert(
   'the inline hook for Rnw applies \\ensuremath{} correctly',
+  .inline.hook.tex(1e4) == '\\ensuremath{10^{4}}',
+  .inline.hook.tex(-Inf) == '\\ensuremath{-\\infty{}}',
   .inline.hook.tex(c(1.2345e10,2* pnorm(-(3:4)))) ==
     "\\ensuremath{1.2345\\times 10^{10}}, 0.0027, \\ensuremath{6.3342\\times 10^{-5}}"
 )
 
+assert(
+  'Infinity and NaN are formatted correctly',
+  identical(format_sci(-Inf), '-\\infty{}'),
+  identical(format_sci(-Inf, 'html'), '-&infin;'),
+  identical(format_sci(-Inf, 'rst'), '-Inf'),
+  identical(format_sci(NaN), 'NaN')
+)
+
 assert(
   'sanitize_fn() warns against spaces in filenames',
   has_warning(sanitize_fn('figure/a b'))
